; ********************************************************
; ** Name:    einsatz.asm                               **
; ** Version: V1.21 vom 14/03/2016                      **
version equ h'121'
; ** Project: Einsatz-Fahrzeug PKW                      **
; **                                                    **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; ** Prozessor: Pic 12F629 / Microchip                  **
; **            1 cycle = 1 us (4 MHz internal Osc)     **
; **                                                    **
; ** IR: RB0 for Input-Signal                           **
; **     TIMER0 for detection of idle-state             **
; **                                                    **
; ** Timer: 0 for detection of idle-state               **
; **        1 for PWM                                   **
; ********************************************************
; Copyright (C) 2003-2016 Sven Brandt

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License version 2
; as published by the Free Software Foundation.
  
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

#ifndef makefile

;#define test                    ; fr Simulation: schnellerer Ablauf
#define test_freezemode         ; Mode fixiert...
#define freezemode d'33'-d'1'   ; .. auf diesen Blink-Mode
;#define out_inv

#endif  ;makefile

;  Mode                                               Nummer Funktion von Ausgang Nummer 
;                                                     Out #  1          2            3          4          5 
;                                                            GPIO 5     GPIO 4       GPIO 2     GPIO 1     GPIO 0
; 5 x Rundum                                              1  Rundum     Rundum       Rundum     Rundum     Rundum  
; 4 x Rundum + Blinker                                    2  Rundum     Rundum       Rundum     Blinker    Rundum  
; 4 x Rundum + Frontlicht                                 3  Rundum     Rundum       Rundum     Rundum     Frontlicht 
; 3 x Rundum + Frontlicht + Blinker                       4  Rundum     Rundum       Rundum     Blinker    Frontlicht 
; 5 x Strobo                                              5  Strobo     Strobo       Strobo     Strobo     Strobo  
; 4 x Strobo + Blinker                                    6  Strobo     Strobo       Strobo     Blinker    Strobo  
; 4 x Strobo + Frontlicht                                 7  Strobo     Strobo       Strobo     Strobo     Frontlicht 
; 3 x Strobo + Frontlicht + Blinker                       8  Strobo     Strobo       Strobo     Blinker    Frontlicht 
; 1 x Wechsel-Frontlicht + 3 x Strobo                     9  Strobo     Strobo       Front-L    Front-R    Strobo  
; 1 x Wechsel-Frontlicht + 2 x Strobo + K-LED             10 Strobo     Strobo       Front-L    Front-R    K-LED  
; 1 x Wechsel-Frontlicht + 1 x Wechsel-Strobo + Strobo    11 Strobo-L-1 Strobo-R-1   Front-L    Front-R    Strobo  
; 1 x Wechsel-Frontlicht + 1 x Wechsel-Strobo + K-LED     12 Strobo-L-1 Strobo-R-1   Front-L    Front-R    K-LED  
; 1 x Wechsel-Strobo + 3 x Strobo                         13 Strobo-L-1 Strobo-R-1   Strobo     Strobo     Strobo  
; 1 x Wechsel-Strobo + 2 x Strobo + Blinker               14 Strobo-L-1 Strobo-R-1   Strobo     Blinker    Strobo  
; 1 x Wechsel-Strobo + 2 x Strobo + Frontlicht            15 Strobo-L-1 Strobo-R-1   Strobo     Strobo     Frontlicht 
; 1 x Wechsel-Strobo + 1 x Strobo + Frontlicht + Blinker  16 Strobo-L-1 Strobo-R-1   Strobo     Blinker    Frontlicht 
; 2 x Wechsel-Strobo + 1 x Strobo                         17 Strobo-L-1 Strobo-R-1   Strobo-L-2 Strobo-R-2 Strobo  
; 2 x Wechsel-Strobo + K-LED                              18 Strobo-L-1 Strobo-R-1   Strobo-L-2 Strobo-R-2 K-LED  
; 2 x Wechsel-Strobo + Frontlicht                         19 Strobo-L-1 Strobo-R-1   Strobo-L-2 Strobo-R-2 Frontlicht 
; Aircraft Anti Collision Lights                          20 Top Light  Bottom Light Wing 1     Wing 2     Tail  
; RTK-6 SL + 2x Strobo                                    21 RTK6 L     RTK6 R       RTK6 Blitz Strobo     Strobo
; RTK-6 SL + Wechselblinker                               22 RTK6 L     RTK6 R       RTK6 Blitz Wechsel 1  Wechsel 2
; 4 x Rundum + 1 x Strobo                                 23 Strobo     Rundum       Rundum     Rundum     Rundum   
; 3 x Rundum + 1x Strobo + Blinker                        24 Strobo     Rundum       Rundum     Blinker    Rundum  
; 2 x K-LED + 3x Strobo                                   25 Strobo     Strobo       Strobo     K-LED      K-LED 
; 5 x K-LED                                               26 K-LED      K-LED        K-LED      K-LED      K-LED   
; 4 x K-LED + 1x Blinker                                  27 K-LED      K-LED        K-LED      Blinker    K-LED   
; Tigis 3er                                               28
; Tigis 5er                                               29
; Delfis                                                  30
; Lschmittelanzeige                                      31
; Lichthupe                                               32 Lichthupe  Rundum       Rundum     Blinker    Strobo
; Drehspiegel langsam                                     33 Dreh1      Dreh2        Dreh3      Blinker    Strobo
; Drehspiegel mittel                                      33 Dreh1      Dreh2        Dreh3      Blinker    Strobo
; Drehspiegel schnell                                     33 Dreh1      Dreh2        Dreh3      Blinker    Strobo

; ********************************************************
; History:
; V1.21 - 14.03.2016 - Fehler Out 3 Drehspiegel behoben
; V1.20a- 19.10.2015 - #define out_inv hinzu fr Einsatz-Mini
; V1.20 - 28.05.2015 - Mode 34/35 hinzu
; V1.19 - 25.03.2015 - Mode 33 hinzu
; V1.18 -            - Mode 32 hinzu
; V1.17 - 13.05.2013 - Mode 30 - 31 hinzu
; V1.16 - 07.12.2012 - Mode 25 korrigiert
; V1.15 - Mode 29 hinzu
; V1.14 - Mode 28 hinzu
; V1.13 - Mode 27 hinzu
; V1.12 - Mode 24/25/26 hinzu, PWM nicht mehr via Tabelle
; V1.11 - Mode 23 hinzu, Button debouncing verbessert (R_CNT1_TGL hinzu)
; V1.10 - Fehler-Korrektur speichern R_MODE
; V1.09 - hinzu: Aircraft Anti Collosion Light
;       - Optimierung der Mode-Nummern --> RAM-Gewinn
; V1.07 - New blink concept
; V1.06 - inverted Strobo corrected
; V1.04 - Output 5: new Modes (inverted Strobo)
; V1.03 - more Modes again
; V1.02 - more Modes (Nov.2006)
; V1.01 - add mode-switching
; V1.00 - created from andreas.asm V1.00
; ********************************************************

 #include <p12f629.inc>
 LIST p=12f629                       ; Controller
 __idlocs h'0000' | version          ; PIC Device ID

 ERRORLEVEL 0,-302                   ; Errorlevel
 __config b'00000111001100'          ; Programmer-Configuration
;           ++---------------- #13-#12 BG1:BG0: Bandgap Calibration bits
;             ||||||||||||        >00 = Lowest bandgap voltage
;             ||||||||||||         11 = Highest bandgap voltage
;             +++------------- #11-#9 Unimplemented: Read as 0
;                +------------ #8: CPD: Data Code Protection bit (2)
;                 ||||||||        >1 = Data memory code protection is disabled
;                 ||||||||         0 = Data memory code protection is enabled
;                 +----------- #7: CP: Code Protection bit
;                  |||||||        >1 = Program Memory code protection is disabled
;                  |||||||         0 = Program Memory code protection is enabled
;                  +---------- #6: BODEN: Brown-out Detect Enable bit
;                   ||||||        >1 = BOD enabled
;                   ||||||         0 = BOD disabled
;                   +--------- #5: MCLRE: GP3/MCLR pin function select
;                    |||||         1 = GP3/MCLR pin function is MCLR
;                    |||||        >0 = GP3/MCLR pin function is digital I/O
;                    +-------- #4: PWRTE: Power-up Timer Enable bitd
;                     ||||         1 = PWRT disabled
;                     ||||        >0 = PWRT enabled
;                     +------- #3: WDTE: Watchdog Timer Enable bit
;                      |||        >1 = WDT enabled
;                      |||         0 = WDT disabled
;                      +++---- #2-#0: FOSC2:FOSC0: Oscillator Selection bits
;                                 >100 = INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin,
;                                        I/O function on GP5/OSC1/CLKIN

; ********************************************************
; **    Files Assignment                                **
; ********************************************************

R_F_SW1            equ  20h          ; Flags for Button
R_CNT_SW1          equ  21h          ; Counter for Button
R_CNT1_TGL         equ  22h          ; Counter1 for Button
R_MODE             equ  23h          ; Register for flashing-mode
R_PWMVALUE         equ  24h          ; PWM-Value
R_PWM              equ  25h          ; Counter for PWM
R_TABLEADRESS_L    equ  26h          ; Fr Tabellensprung

R_STATUS           equ  2ah          ; Status-Flags Frontlich Wechsel
R_STATUS_FRONT     equ  2bh          ; Status-Flags Frontlicht (Lichthupe)

R_CNT_F1           equ  30h          ; Counter for FLASH1
R_FLAG_F1          equ  31h          ; FLAGS for FLASH1
R_CNT_F2           equ  32h          ; Counter for FLASH2
R_FLAG_F2          equ  33h          ; FLAGS for FLASH2
R_CNT_F3           equ  34h          ; Counter for FLASH3
R_FLAG_F3          equ  35h          ; FLAGS for FLASH3
R_CNT_F4           equ  36h          ; Counter for FLASH4
R_CNT_F4_2         equ  37h          ; Counter for FLASH4
R_FLAG_F4          equ  38h          ; FLAGS for FLASH4
R_CNT_F5           equ  39h          ; Counter for FLASH5
R_FLAG_F5          equ  3ah          ; FLAGS for FLASH5
                   
R_FLAG_2           equ  3bh          ; div. Flags (Register wird nicht in LOOP gelscht)
#define F_WASSER_BLINK             R_FLAG_2,0 ; #0: Wasserstand Blinken 

R_CNT_WASSER_1     equ  40h          ; Couter TimeDelay Wasserstand
R_CNT_WASSER_2     equ  41h          ; Couter TimeDelay Wasserstand
R_CNT_LICHTHUPE_1  equ  42h          ; Counter Lichthupe
R_CNT_LICHTHUPE_2  equ  43h          ; Counter Lichthupe

R_FLAG_BLINK_0     equ  50h          ; Flags Blink-Mode (Register wird in jeder LOOP gelscht, Flags dann je nach Mode gesetzt)
#define F_OUT1_2_3_BLINK           R_FLAG_BLINK_0,7 ; 
#define F_OUT1_2_BLINK             R_FLAG_BLINK_0,6 ; 
#define F_OUT3_4_BLINK             R_FLAG_BLINK_0,5 ; 
#define F_OUT2_BLINK               R_FLAG_BLINK_0,4 ; 
#define F_OUT4_BLINK               R_FLAG_BLINK_0,3 ; 
#define F_OUT5_BLINK               R_FLAG_BLINK_0,2 ; 
#define F_OUT4_5_WECHSEL           R_FLAG_BLINK_0,1 ; 
;#define F_                         R_FLAG_BLINK_0,0 ; 

R_FLAG_BLINK_1     equ  51h          ; Flags Blink-Mode (Register wird in jeder LOOP gelscht, Flags dann je nach Mode gesetzt)
#define F_OUT4_BLINKER             R_FLAG_BLINK_1,7 ; 
#define F_OUT5_FRONTLIGHT          R_FLAG_BLINK_1,6 ; 
#define F_OUT3_4_FRONTLIGHT        R_FLAG_BLINK_1,5 ; 
#define F_OUT1_KLED                R_FLAG_BLINK_1,4 ; 
#define F_OUT2_KLED                R_FLAG_BLINK_1,3 ; 
#define F_OUT3_KLED                R_FLAG_BLINK_1,2 ; 
#define F_OUT4_KLED                R_FLAG_BLINK_1,1 ; 
#define F_OUT5_KLED                R_FLAG_BLINK_1,0 ; 

R_FLAG_BLINK_2     equ  52h          ; Flags Blink-Mode (Register wird in jeder LOOP gelscht, Flags dann je nach Mode gesetzt)
#define F_OUT1_5_AIRCRAFT          R_FLAG_BLINK_2,7 ; 
#define F_OUT1_2_3_TIGIS           R_FLAG_BLINK_2,6 ; 
#define F_OUT1_2_3_4_5_TIGIS       R_FLAG_BLINK_2,5 ; 
#define F_WASSERANZEIGE            R_FLAG_BLINK_2,4 ; 
#define F_DELFIS                   R_FLAG_BLINK_2,3 ; aus Tigis wird Delfis (F_OUT1_2_3_4_5_TIGIS bentigt)
#define F_OUT12_STROBO_WECHSEL     R_FLAG_BLINK_2,2 ; Out 1/2 im Wechsel Strobo
#define F_OUT34_STROBO_WECHSEL     R_FLAG_BLINK_2,1 ; Out 3/4 im Wechsel Strobo
#define F_LICHTHUPE                R_FLAG_BLINK_2,0 ; wird mir F_OUT5_FRONTLIGHT zur Lichthupe

R_FLAG_BLINK_3     equ  53h          ; Flags Blink-Mode (Register wird in jeder LOOP gelscht, Flags dann je nach Mode gesetzt)
#define F_OUT1_STROBO              R_FLAG_BLINK_3,7 ; Strobo-Mode fr Ausgang 1 (Doppelblitze)
#define F_OUT2_STROBO              R_FLAG_BLINK_3,6 ; Strobo-Mode fr Ausgang 2 (Doppelblitze)
#define F_OUT345_STROBO            R_FLAG_BLINK_3,5 ; Strobo-Mode fr Ausgang 3-5 (Doppelblitze)
#define F_OUT123_LL                R_FLAG_BLINK_3,4 ; OUT 123 Lauflicht (wie Drehspiegel)
;#define F_                         R_FLAG_BLINK_3,3 ; 
;#define F_                         R_FLAG_BLINK_3,2 ; 
;#define F_                         R_FLAG_BLINK_3,1 ; 
;#define F_                         R_FLAG_BLINK_3,0 ; 

;   --------------------------------------------------------------------------
;   --- Port-Definitionen
;   --------------------------------------------------------------------------
PORTA                        EQU     H'0005' 

#define IO_OUT1      PORTA,5        ;Layout Definition, Output 
#define IO_OUT2      PORTA,4        ;Layout Definition, Output 
#define IO_OUT3      PORTA,2        ;Layout Definition, Output 
#define IO_OUT4      PORTA,1        ;Layout Definition, Output 
#define IO_OUT5      PORTA,0        ;Layout Definition, Output 
#define IO_BUTTON    PORTA,3        ;Layout Definition, Input 

#ifdef out_inv
 #define out_1_on  bcf IO_OUT1
 #define out_2_on  bcf IO_OUT2
 #define out_3_on  bcf IO_OUT3
 #define out_4_on  bcf IO_OUT4
 #define out_5_on  bcf IO_OUT5
 #define out_1_off bsf IO_OUT1
 #define out_2_off bsf IO_OUT2
 #define out_3_off bsf IO_OUT3
 #define out_4_off bsf IO_OUT4
 #define out_5_off bsf IO_OUT5
#else ; nicht invertiert
 #define out_1_on  bsf IO_OUT1
 #define out_2_on  bsf IO_OUT2
 #define out_3_on  bsf IO_OUT3
 #define out_4_on  bsf IO_OUT4
 #define out_5_on  bsf IO_OUT5
 #define out_1_off bcf IO_OUT1
 #define out_2_off bcf IO_OUT2
 #define out_3_off bcf IO_OUT3
 #define out_4_off bcf IO_OUT4
 #define out_5_off bcf IO_OUT5
#endif ; out_inv

;   --------------------------------------------------------------------------
;   --- Parameter
;   --------------------------------------------------------------------------
;Timing:
T_FLASH1    equ d'250'        ; (1..255) duration of long off-time for FLASH1
T_BREAK1    equ d'45'         ; (1..255) duration of short off-time for FLASH1

T_FLASH2    equ d'255'        ; (1..255) duration of long off-time for FLASH2
T_BREAK2    equ d'55'         ; (1..255) duration of short off-time for FLASH2

T_FLASH3    equ d'246'        ; (1..255) duration of long off-time for FLASH3
T_BREAK3    equ d'50'         ; (1..255) duration of short off-time for FLASH3

T_FLASH4    equ d'240'        ; (1..255) duration of for FLASH4
T_FLASH4_2  equ d'200'        ; (1..255) duration of for FLASH4 - Blinker
T_BREAK4    equ d'30'         ; (1..255) duration of short off-time for FLASH4

T_FLASH5    equ d'243'        ; (1..255) duration of for FLASH5
T_BREAK5    equ d'40'         ; (1..255) duration of short off-time for FLASH5

T_FLASH6    equ d'125'        ; (1..255) duration of for FLASH6 (aircraft)
T_BREAK6    equ d'83'         ; (1..255) duration of short off-time for FLASH6 (aircraft)

T_BREAK7a   equ d'30'         ; (1..255) duration of for FLASH7 (RTK-6)
T_BREAK7b   equ d'100'        ; (1..255) 1. duration of short off-time for FLASH7 (RTK-6)
T_BREAK7c   equ d'140'        ; (1..255) 2. duration of short off-time for FLASH7 (RTK-6)

T_FLASH8    equ d'160'        ; (1..255) duration of for FLASH8 (Wechselblinken), 1sec.

T_FLASH     equ d'10'         ; duration of a flash

T_FLASH_I1  equ d'70'         ; K-LED: duration of a flash #1
T_FLASH_I2  equ d'20'         ; K-LED: duration of a flash #2

T_BREAK1_I  equ d'115'        ; K-LED: (1..255) duration of short off-time for Out#4
T_FLASH1_I  equ d'103'        ; K-LED: (1..255) duration of for Out#4

T_BREAK2_I  equ d'114'        ; K-LED: (1..255) duration of short off-time for Out#4
T_FLASH2_I  equ d'102'        ; K-LED: (1..255) duration of for Out#4

T_BREAK3_I  equ d'108'        ; K-LED: (1..255) duration of short off-time for Out#4
T_FLASH3_I  equ d'98'         ; K-LED: (1..255) duration of for Out#4

T_BREAK4_I  equ d'113'        ; K-LED: (1..255) duration of short off-time for Out#4
T_FLASH4_I  equ d'103'        ; K-LED: (1..255) duration of for Out#4

T_BREAK5_I  equ d'110'        ; K-LED: (1..255) duration of short off-time for Out#5
T_FLASH5_I  equ d'100'        ; K-LED: (1..255) duration of for Out#5

T_BREAK28a   equ d'2'         ; (1..255) off-time 1 for FLASH28 (Tigis)
T_BREAK28c   equ d'40'        ; (1..255) off-time 3 for FLASH28 (Tigis)
T_BREAK28d   equ d'100'       ; (1..255) off-time 3 for FLASH28 (Tigis)
T_BREAK28e   equ d'150'       ; (1..255) off-time 4 for FLASH28 (Tigis))
T_FLASH28a   equ d'30'
T_FLASH28b   equ d'50'
T_FLASH28c   equ d'30'

T_BREAK29a   equ d'2'         ; (1..255) off-time 1 for FLASH28 (Tigis)
T_BREAK29c   equ d'40'        ; (1..255) off-time 3 for FLASH28 (Tigis)
T_BREAK29d   equ d'100'       ; (1..255) off-time 3 for FLASH28 (Tigis)
T_BREAK29e   equ d'150'       ; (1..255) off-time 4 for FLASH28 (Tigis))
T_FLASH29a   equ d'30'
T_FLASH29b   equ d'50'
T_FLASH29c   equ d'30'

;T_LL         equ d'40'       ; Step-Time for Lauflicht

CONST_CNT_LICHTHUPE_WARTE  equ d'120' ; Wartezeit Lichthupe
CONST_CNT_LICHTHUPE_HELL   equ d'4'   ; Hell-Zeit 1 Lichthupe
CONST_CNT_LICHTHUPE_HELL2  equ d'2'   ; Hell-Zeit 2 Lichthupe
CONST_CNT_LICHTHUPE_DUNKEL equ d'2'   ; Dunkel-Zeit zwischen den 2 Hupen

PWM_MIN     equ d'30'         ; minimum Value for PWM

ANZ_MODI    equ d'35'         ; number of modes / Anzahl Modi

; Werte zur Berechnung der Sprung-Adressen fr die Tabelle it den Modi-Parametern
#define     Table_Adress_Start   0x59    ; Erste Zeile der Tabelle (nur LOW Byte)
#define     Table_Length         d'40'   ; Anzahl der Tabellenzeilen


; ********************************************************
; **  Reset Vector                                      **
; ********************************************************

  org  0000h                           ; Position (Reset Vector)
        goto start
 
; ********************************************************
; **  Interupt Service Routine ISR                      **
; ********************************************************
  org 0004h                            ; Interupt Vector
        retfie




; ********************************************************
; **  Programm                                          **
; ********************************************************
 org   0006h                           ; Memory-Position
;   -----------------------------
;   --- clear memory $20..$59 ---
;   -----------------------------
start
        movlw h'20'                    ; memory start adress
        movwf FSR
next_clr
        clrf INDF                      ; clear
        incf FSR,F
        movlw h'60'                    ; Anzahl der Adressen
        subwf FSR,W                    ; W = FSR - W = FSR - h'5F'
        btfss STATUS,C                 ; if C=0, -> FSR < h'5F'
         goto next_clr                  ; next bit

;   --------------------
;   --- start values ---
;   --------------------
        bsf R_STATUS,4
        bsf R_STATUS_FRONT,5

        movlw h'ff'                     ; clear counter
        movwf R_CNT_SW1                 ; .. to FF

        movlw d'1'                       ; Reload Counter mit "1" fr alle States ohne Wartezeit
         movwf R_CNT_LICHTHUPE_1         ; .. laden in Counter 2
         movwf R_CNT_LICHTHUPE_2         ; .. laden in Counter 2

        bsf R_F_SW1,0                  ; set flag -> idle for mode switch needed

; ********************************************************
; **  Konfiguration PORTS                               **
; ********************************************************

        bsf STATUS,RP0                 ; -> BANK 1
        clrf TRISIO                    ; TRIS-Reg. (all Output)
        bsf TRISIO,3                   ; Input
        bcf STATUS,RP0                 ; -> BANK 0
        clrf GPIO

; ********************************************************
; **  Konfiguration COMPARATOR                          **
; ********************************************************

        movlw   B'00000111'
;                 +--------- #7: Unimplemented: Read as 0
;                  +-------- #6: COUT: Comparator Output bit
;                   |||| |       When CINV=0  1 = VIN+ > VIN-
;                   |||| |                    0 = VIN+ < VIN-
;                   |||| |       When CINV=1  0 = VIN+ > VIN-
;                   |||| |                    1 = VIN+ < VIN-
;                   +------- #5: Unimplemented: Read as 0
;                    +------ #4: CINV: Comparator Output Inversion bit
;                     || |       1 = Output inverted
;                     || |       0 = Output not inverted
;                     +----- #3: CIS: Comparator Input Switch bit
;                      | |       When CM2:CM0 = 110 or 101:
;                      | |       1 = VIN- connects to CIN+
;                      | |       0 = VIN- connects to CIN-
;                      +++-- #2-#0: CM2:CM0: Comparator Mode bits
;                               >111 = Comparator Off

       movwf CMCON                     ; CNCON-Reg. konfigurieren

; ********************************************************
; **  Konfiguration TIMER / WDT / Pull-Ups              **
; ********************************************************

        movlw   B'11000000'
;                 +--------- #7: GPPU: GPIO Pull-up Enable bit
;                  ||||| |      >1 = GPIO pull-ups are disabled
;                  ||||| |       0 = GPIO pull-ups are enabled by individual port latch values
;                  +-------- #6: INTEDG: Interrupt Edge Select bit
;                   |||| |      >1 = Interrupt on rising edge of GP2/INT pin
;                   |||| |       0 = Interrupt on falling edge of GP2/INT pin
;                   +------- #5: T0CS: TMR0 Clock Source Select bit
;                    ||| |       1 = Transition on GP2/T0CKI pin
;                    ||| |      >0 = Internal instruction cycle clock (CLKOUT)
;                    +------ #4: T0SE: TMR0 Source Edge Select bit
;                     || |       1 = Increment on high-to-low transition on GP2/T0CKI pin
;                     || |      >0 = Increment on low-to-high transition on GP2/T0CKI pin
;                     +----- #3: PSA: Prescaler Assignment bit
;                      | |       1 = Prescaler is assigned to the WDT
;                      | |      >0 = Prescaler is assigned to the TIMER0 module
;                      +++-- #2-#0: PS2:PS0: Prescaler Rate Select bits
;                                 Bit |TMR0 Rate|WDT Rate
;                                 ----+---------+--------
;                               > 000 | 1 : 2   | 1 : 1
;                                 001 | 1 : 4   | 1 : 2
;                                 010 | 1 : 8   | 1 : 4
;                                 011 | 1 : 16  | 1 : 8
;                                 100 | 1 : 32  | 1 : 16
;                                 101 | 1 : 64  | 1 : 32
;                                 110 | 1 : 128 | 1 : 64
;                                 111 | 1 : 256 | 1 : 128

        bsf STATUS,RP0                 ; -> BANK 1
        movwf OPTION_REG               ; OPTION-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'00110001'
;                 +--------- #7: Unimplemented: Read as 0
;                  +-------- #6: TMR1GE: Timer1 Gate Enable bit
;                   |||| |      >If TMR1ON = 0: This bit is ignored
;                   |||| |       If TMR1ON = 1:
;                   |||| |       1 = Timer1 is on if T1G pin is low
;                   |||| |       0 = Timer1 is on
;                   ++------ #5-#4: T1CKPS1:T1CKPS0: Timer1 Input Clock Prescale Select bits
;                    ||| |      >11 = 1:8 Prescale Value
;                    ||| |       10 = 1:4 Prescale Value
;                    ||| |       01 = 1:2 Prescale Valueue
;                    ||| |       00 = 1:1 Prescale Value
;                     +----- #3: T1OSCEN: LP Oscillator Enable Control bit
;                      | |       If INTOSC without CLKOUT oscillator is active:
;                      | |       1 = LP oscillator is enabled for Timer1 clock
;                      | |      >0 = LP oscillator is off
;                      | |       Else: This bit is ignored
;                      +---- #2: T1SYNC: Timer1 External Clock Input Synchronization Control bit
;                       ||      >if TMR1CS = 0: This bit is ignored. Timer1 uses the internal clock.
;                       +--- #1: TMR1CS: Timer1 Clock Source Select bit
;                        |       1 = External clock from T1OSO/T1CKI pin (on the rising edge)
;                        |      >0 = Internal clock (FOSC/4)
;                        +-- #0: TMR1ON: Timer1 On bit
;                               >1 = Enables Timer1
;                                0 = Stops Timer1

        movwf T1CON                    ; T1CON-Reg. konfigurieren
        clrf TMR1L
        clrf TMR1H

; ********************************************************
; **  Konfiguration INTERRUPT                           **
; ********************************************************

        movlw   B'00000000'
;                 +--------- #7: GIE: Global Interrupt Enable bit
;                  |||||||       1 = Enables all un-masked interrupts
;                  |||||||      >0 = Disables all interrupts
;                  +-------- #6: PEIE: Peripherial Interrupt Enable bit
;                   ||||||       1 = Enables all unmasked peripherial interrupts
;                   ||||||      >0 = Disables all peripherial interrupts
;                   +------- #5: T0IE: TMR0 Overflow Interrupt Enable bit
;                    |||||       1 = Enables the TMR0 interrupt
;                    |||||      >0 = Disables the TMR0 interrupt
;                    +------ #4: INTE: GP2/INT External Interrupt Enable bit
;                     ||||       1 = Enables the GP2/INT external interrupt
;                     ||||      >0 = Disables the GP2/INT external interrupt
;                     +----- #3: GPIE: Port Change Interrupt Enable bit
;                      |||       1 = Enables the GPIO port change interrupt
;                      |||      >0 = Disables the GPIO port change interrupt
;                      +---- #2: T0IF: TMR0 overflow interrupt flag bit
;                       ||       1 = TMR0 has overflowed (must be cleared in software)
;                       ||       0 = TMR0 did not overflow
;                       +--- #1: INTF: GP2/INT External Interrupt Flag bit
;                        |       1 = The GP2/INT external interrupt occurred (must be cleared in software)
;                        |       0 = The GP2/INT external interrupt did not occur
;                        +-- #0: GPIF: Port Change Interrupt Flag bit
;                                1 = When at least one of the GP5:GP0 pins changed state
;                                    (must be cleared in software)
;                                0 = None of the GP5:GP0 pins have changed state

        movwf INTCON                   ; INTCON-Reg. konfigurieren

        movlw   B'00000000'
;                 +--------- #7: EEIE: EE Write Complete Interrupt Enable Bit
;                  |||||||       1 = Enables the EE write complete interrupt
;                  |||||||      >0 = Disables the EE write complete interrupt
;                  +-------- #6: ADIE: A/D Converter Interrupt Enable bit (PIC12F675 only)
;                                1 = Enables the A/D converter interrupt
;                               >0 = Disables the A/D converter interrupt
;                   +------- #5: Unimplemented: Read as 0
;                    +------ #4: Unimplemented: Read as 0
;                     +----- #3: CMIE: Comparator Interrupt Enable bit
;                                1 = Enables the comparator interrupt
;                               >0 = Disables the comparator interrupt
;                      +---- #2: Unimplemented: Read as 0
;                       +--- #1: Unimplemented: Read as 0
;                        +-- #0: TMR1IE: TMR1 Overflow Interrupt Enable bit
;                                1 = Enables the TMR1 overflow interrupt
;                               >0 = Disables the TMR1 overflow interrupt

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PIE1                     ; PIE1-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0


; ********************************************************
; **  read R_MODE from EEPROM                           **
; ********************************************************
        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'31'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

        movlw ANZ_MODI                 ; check mode
        subwf R_MODE,W                 ; W = R_MODE - W 
        btfss STATUS,C                 ; if C=0, -> R_MODE < wert
         goto loop                     ; ...then jump

        clrf R_MODE

; ********************************************************
; **  Main Loop                                         **
; ********************************************************
loop    clrwdt                         ; clear Watchdog-Timer

#ifdef test_freezemode
        movlw freezemode  ; test
        movwf R_MODE      ; test             ; W = R_MODE - W
#endif ;test

; ***** check switch mode *****
check_sw1
        nop
        decfsz R_CNT1_TGL,F            ; increment Counter 1 for debouncing switch
         goto loop_0

        btfsc IO_BUTTON
         goto check_sw1_1

        ;*** SW1 is low ***
check_sw1_0
        movlw d'10'
        addwf R_CNT_SW1,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_sw1_2              ; if R_CNT_SW1 > 255 then jump
        goto loop_0

        ;*** SW1 is high (active) ***
check_sw1_1
        btfsc R_F_SW1,0                ; .. if flag
         goto loop_0                   ; .. then jump

        movlw d'10'
        subwf R_CNT_SW1,F
        btfss STATUS,C                 ; check for CARRY
         goto check_sw1_3              ; if R_CNT_SW1 < 0 then jump
        goto loop_0

        ;*** SW1 is not active ***
check_sw1_2
        bcf R_F_SW1,0                  ; clear flag
        movlw h'ff'                    ; set counter
        movwf R_CNT_SW1                ; .. to FF
        goto loop_0

        ;*** SW1 is active ***
check_sw1_3 
        bsf R_F_SW1,0                  ; set flag
        clrf R_CNT_SW1                 ; clear counter
        call change_mode               ; "change mode"
        goto loop_0

; ***** create 2ms slot *****
loop_0
#ifndef test
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         goto loop
        clrf TMR1H                     ; .. then reset to ZERO
        ; *** every 2 ms ***
#endif ;test

; ************** Mode-Definitionen laden *********************************
        movf R_MODE,W
        addlw Table_Adress_Start
        movwf R_TABLEADRESS_L
        call tableread                 ; Table Read execute
        movwf R_FLAG_BLINK_0           ; Ablegen in das Register
        clrf PCLATH

        movf R_MODE,W
        addlw Table_Adress_Start + Table_Length
        movwf R_TABLEADRESS_L
        call tableread                 ; Table Read execute
        movwf R_FLAG_BLINK_1           ; Ablegen in das Register
        clrf PCLATH

        movf R_MODE,W
        addlw Table_Adress_Start + Table_Length + Table_Length
        movwf R_TABLEADRESS_L
        call tableread                 ; Table Read execute
        movwf R_FLAG_BLINK_2           ; Ablegen in das Register
        clrf PCLATH

        movf R_MODE,W
        addlw Table_Adress_Start + Table_Length + Table_Length + Table_Length
        movwf R_TABLEADRESS_L
        call tableread                 ; Table Read execute
        movwf R_FLAG_BLINK_3           ; Ablegen in das Register
        clrf PCLATH

; ************** Blink-Mode ausfhren *********************************

        btfsc F_OUT1_2_BLINK
         call out1_2_blink
        btfsc F_OUT2_BLINK
         call out2_blink
        btfsc F_OUT3_4_BLINK
         call out3_4_blink
        btfsc F_OUT4_BLINK
         call out4_blink
        btfsc F_OUT5_BLINK
         call out5_blink
        btfsc F_OUT4_BLINKER
         call out4_blinker
        btfsc F_OUT5_FRONTLIGHT
         call out5_frontlight
        btfsc F_OUT3_4_FRONTLIGHT
         call out3_4_frontlight
        btfsc F_OUT1_5_AIRCRAFT
         call out1_5_aircraft
        btfsc F_OUT1_2_3_BLINK
         call out1_2_3_blink
        btfsc F_OUT4_5_WECHSEL
         call out4_5_wechsel
        btfsc F_OUT1_KLED
         call out1_kled
        btfsc F_OUT2_KLED
         call out2_kled
        btfsc F_OUT3_KLED
         call out3_kled
        btfsc F_OUT4_KLED
         call out4_kled
        btfsc F_OUT5_KLED
         call out5_kled
        btfsc F_OUT1_2_3_TIGIS
         call out1_2_3_tigis
        btfsc F_OUT1_2_3_4_5_TIGIS
         call out1_2_3_4_5_tigis
        btfsc F_WASSERANZEIGE
         call wasseranzeige
        btfsc F_OUT123_LL
         call out123_ll

        goto loop                      ; endless loop

; ********************************************************
; **  SUB - Routines                                    **
; ********************************************************
;*********************************
;** Routine: change_mode        **
;** change & check R_MODE and   **
;** save into EEPROM $31        **
;*********************************
change_mode
        incf R_MODE,F                  ; +1

        movlw ANZ_MODI                 ; check mode
        subwf R_MODE,W                 ; W = R_MODE - W 
        btfsc STATUS,C                 ; if C=1, -> R_MODE >= wert
        clrf R_MODE                    ; then RESET to ZERO

        ; Mode-Wechsel --> Reset Flags
        clrf R_STATUS
        bsf R_STATUS,4

        clrf R_STATUS_FRONT
        bsf R_STATUS_FRONT,5

        clrf R_PWMVALUE           ; PWM-Value from Table-Read
        clrf R_PWM                ; Counter for PWM
        clrf R_CNT_F1             ; Counter for FLASH1
        clrf R_FLAG_F1            ; FLAGS for FLASH1
        clrf R_CNT_F2             ; Counter for FLASH2
        clrf R_FLAG_F2            ; FLAGS for FLASH2
        clrf R_CNT_F3             ; Counter for FLASH3
        clrf R_FLAG_F3            ; FLAGS for FLASH3
        clrf R_CNT_F4             ; Counter for FLASH4
        clrf R_CNT_F4_2           ; Counter for FLASH4
        clrf R_FLAG_F4            ; FLAGS for FLASH4
        clrf R_CNT_F5             ; Counter for FLASH5
        clrf R_FLAG_F5            ; FLAGS for FLASH5

        movlw d'1'                       ; Reload Counter mit "1" fr alle States ohne Wartezeit
         movwf R_CNT_LICHTHUPE_1         ; .. laden in Counter 1
         movwf R_CNT_LICHTHUPE_2         ; .. laden in Counter 2


        ; save into EEPROM $31
        bcf INTCON,GIE                 ; disable IRs
;       **** write R_MODE ****
        movf R_MODE,W
        bsf STATUS,RP0                 ; -> BANK 1
        bsf EECON1,WREN                ; enable write
        movwf EEDATA                   ; R_MODE into EEDATA
        movlw h'31'
        movwf EEADR                    ; define EEADR (Adress)
        movlw 0x55
        movwf EECON2
        movlw 0xAA
        movwf EECON2
        bsf EECON1,WR                  ; start writing
        btfsc EECON1,WR
         goto $-1                      ; wait for finish

        bcf EECON1,WREN                ; disable write
        bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs

        return

;*********************************
;** Routine: out1_2_3_blink       **
;*********************************
; Ausgang 1/2/3 blinken. (RTK)
out1_2_3_blink
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
         out_1_off                     ; .. FLASH1 off
         out_2_off                     ; .. FLASH2 off
         out_3_off                     ; .. FLASH3 off

        movlw T_BREAK7a                ; Time between 2 Flashes (time_a)
        btfsc R_FLAG_F1,1              ; if short Time_B needed 
         movlw T_BREAK7b               ; Time between 2 Flashes (time_b)
        btfsc R_FLAG_F1,2              ; if short Time_C needed 
         movlw T_BREAK7c               ; Time between 2 Flashes (time_c)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; R_CNT >= T_BREAKx
        bcf STATUS,C                   ; Carry lschen fr RLF
        rlf R_FLAG_F1,F                ; shift R_FLAG_F1 
        btfsc R_FLAG_F1,3              ; berlauf nach RLF    
         bsf R_FLAG_F1,0               ; .. abfangen 

        movf R_FLAG_F1,F               ; if R_FLAG_F1 is     
        btfsc STATUS,Z                 ; .. ZERO (no Flag)
         bsf R_FLAG_F1,0               ; .. set flag #0

        ; LED on (wenn R_CNT >= T_BREAKx)
        btfsc R_FLAG_F1,0
         out_1_on                      ; .. FLASH1 on
        btfsc R_FLAG_F1,1
         out_2_on                      ; .. FLASH2 on
        btfsc R_FLAG_F1,2
         out_3_on                      ; .. FLASH2 on

        clrf R_CNT_F1

        return

;*********************************
;** Routine: out1_2_blink       **
;*********************************
; Ausgang 1 blinken. Wenn F_OUT12_STROBO_WECHSEL gesetzt, dann auch Ausgang 2 blinken (Strobo im Wechsel)
; ist F_OUT1_STROBO gesetzt, dann wird im Strobo-Mode geblitzt (Doppelblitz)
out1_2_blink
        btfss F_LICHTHUPE
         goto out1_blink_skip_lh       ; keine Lichthupe  -> Skip Routine
 
        ; Lichthupen-Funktion - Spezieller Ablauf fr Mode 32 
        btfsc R_STATUS_FRONT,0         ; nur in R_STATUS_FRONT#0..
         out_1_off                     ; .. FLASH1 off
        btfsc R_STATUS_FRONT,0         ; nur in R_STATUS_FRONT#0..
         return                        ; .. Routine nicht ausfhren

out1_blink_skip_lh
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
        btfss R_FLAG_F1,1
         out_1_off                     ; .. FLASH1 off
        btfsc R_FLAG_F1,1
         out_2_off                     ; .. FLASH2 off

        movlw T_FLASH1                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F1,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK1                ; Time between 2 Flashes (short time)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F1,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        btfss F_OUT12_STROBO_WECHSEL   ; if not "Strobo R/L-Mode"     
         bcf R_FLAG_F1,1               ; .. clear flag for R-L selection 
        movlw b'00000001'
        btfss F_OUT1_STROBO            ; if not "Strobo"-Mode     
         movwf R_FLAG_F1               ; .. set flag for long time/ clear R-L selection

        btfss R_FLAG_F1,1
         out_1_on                      ; .. FLASH1 on
        btfsc R_FLAG_F1,1
         out_2_on                      ; .. FLASH2 on

        clrf R_CNT_F1

        return




;*********************************
;** Routine: out3_4_blink       **
;*********************************
; Ausgang 3 blinken. Wenn F_OUT34_STROBO_WECHSEL gesetzt, dann auch Ausgang 4 blinken (Strobo im Wechsel)
; ist F_OUT345_STROBO gesetzt, dann wird im Strobo-Mode geblitzt (Doppelblitz)
out3_4_blink
        incf R_CNT_F3,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F3,W               ; .. R_CNT_F3 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
        btfss R_FLAG_F3,1
         out_3_off                     ; .. FLASH1 off
        btfsc R_FLAG_F3,1
         out_4_off                     ; .. FLASH2 off

        movlw T_FLASH3                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F3,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK3                ; Time between 2 Flashes (short time)
        subwf R_CNT_F3,W               ; if R_CNT_F3 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F3,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        btfss F_OUT34_STROBO_WECHSEL   ; if not "Strobo R/L-Mode"     
         bcf R_FLAG_F3,1               ; .. clear flag for R-L selection 
        movlw b'00000001'
        btfss F_OUT345_STROBO          ; if not "Strobo"-Mode     
         movwf R_FLAG_F3               ; .. set flag for long time/ clear R-L selection

        btfss R_FLAG_F3,1
         out_3_on                      ; .. FLASH1 on
        btfsc R_FLAG_F3,1
         out_4_on                      ; .. FLASH2 on

        clrf R_CNT_F3

        return

;*********************************
;** Routine: out2_blink         **
;*********************************
; Ausgang 2 blinken. 
; ist F_OUT2_STROBO gesetzt, dann wird im Strobo-Mode geblitzt (Doppelblitz)
out2_blink
        btfss F_LICHTHUPE
         goto out2_blink_skip_lh       ; keine Lichthupe  -> Skip Routine

        ; Lichthupen-Funktion - Spezieller Ablauf fr Mode 32
        btfsc R_STATUS_FRONT,0         ; nur in R_STATUS_FRONT#0..
         out_2_off                     ; .. FLASH2 off
        btfsc R_STATUS_FRONT,0         ; nur in R_STATUS_FRONT#0..
         return                        ; .. Routine nicht ausfhren

out2_blink_skip_lh
        incf R_CNT_F2,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F2,W               ; .. R_CNT_F2 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
;       btfss R_FLAG_F2,1
         out_2_off                     ; .. FLASH2 off
;       btfsc R_FLAG_F2,1
;        out_2_off                     ; .. FLASH2 off

        movlw T_FLASH2                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F2,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK2                ; Time between 2 Flashes (short time)
        subwf R_CNT_F2,W               ; if R_CNT_F2 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F2,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
;       btfss F_OUT12_STROBO_WECHSEL   ; if not "Strobo R/L-Mode"     
;        bcf R_FLAG_F2,1               ; .. clear flag for R-L selection 
        movlw b'00000001'
        btfss F_OUT2_STROBO            ; if not "Strobo"-Mode     
         movwf R_FLAG_F2               ; .. set flag for long time/ clear R-L selection

 ;      btfss R_FLAG_F2,1
         out_2_on                      ; .. FLASH2 on
 ;      btfsc R_FLAG_F2,1
 ;       out_2_on                      ; .. FLASH2 on

        clrf R_CNT_F2

        return

;*********************************
;** Routine: out4_blink         **
;*********************************
; Ausgang 4 blinken. 
; ist F_OUT345_STROBO gesetzt, dann wird im Strobo-Mode geblitzt (Doppelblitz)
out4_blink
        incf R_CNT_F4,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F4,W               ; .. R_CNT_F4 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
;       btfss R_FLAG_F4,1
         out_4_off                     ; .. FLASH1 off
;       btfsc R_FLAG_F4,1
;        out_4_off                     ; .. FLASH2 off

        movlw T_FLASH4                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F4,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK4                ; Time between 2 Flashes (short time)
        subwf R_CNT_F4,W               ; if R_CNT_F4 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F4,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
;       btfss F_OUT34_STROBO_WECHSEL   ; if not "Strobo R/L-Mode"     
;        bcf R_FLAG_F4,1               ; .. clear flag for R-L selection 
        movlw b'00000001'
        btfss F_OUT345_STROBO          ; if not "Strobo"-Mode    
         movwf R_FLAG_F4               ; .. set flag for long time/ clear R-L selection

 ;      btfss R_FLAG_F4,1
         out_4_on                      ; .. FLASH1 on
 ;      btfsc R_FLAG_F4,1
 ;       out_4_on                      ; .. FLASH2 on

        clrf R_CNT_F4

        return

;*********************************
;** Routine: out5_blink         **
;*********************************
; Ausgang 5 blinken. 
; ist F_OUT345_STROBO gesetzt, dann wird im Strobo-Mode geblitzt (Doppelblitz)
out5_blink
        incf R_CNT_F5,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F5,W               ; .. R_CNT_F5 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; OUT5 off (wenn R_CNT >= T_FLASH)
        out_5_off                      ; .. OUT5 off

        movlw T_FLASH5                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F5,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK5                ; Time between 2 Flashes (short time)
        subwf R_CNT_F5,W               ; if R_CNT_F5 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F5,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        movlw b'00000001'
        btfss F_OUT345_STROBO          ; if not "Strobo"-Mode    
         movwf R_FLAG_F5               ; .. set flag for long time/ clear R-L selection

        out_5_on                       ; .. OUT5 on

        clrf R_CNT_F5

        return

;**********************************
;** Routine: out1_kled           **
;**********************************
; Ausgang 1 blinken K-LED Muster. 
out1_kled
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH_I2               ; duration of a inverted flash #2 
        btfss R_FLAG_F1,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_FLASH_I1              ; duration of a inverted flash #1
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off
        out_1_off                      ; .. LED off

        movlw T_FLASH1_I               ; Time between 2 Flashes (long time)
        btfss R_FLAG_F1,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK1_I              ; Time between 2 Flashes (short time)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F1,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        out_1_on                       ; .. LED on
        clrf R_CNT_F1

        return

;**********************************
;** Routine: out2_kled           **
;**********************************
; Ausgang 2 blinken K-LED Muster. 
out2_kled
        incf R_CNT_F2,F                ; +1

        movlw T_FLASH_I2               ; duration of a inverted flash #2 
        btfss R_FLAG_F2,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_FLASH_I1              ; duration of a inverted flash #1
        subwf R_CNT_F2,W               ; .. R_CNT_F2 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off
        out_2_off                      ; .. LED off

        movlw T_FLASH2_I               ; Time between 2 Flashes (long time)
        btfss R_FLAG_F2,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK2_I              ; Time between 2 Flashes (short time)
        subwf R_CNT_F2,W               ; if R_CNT_F2 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F2,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        out_2_on                       ; .. LED on
        clrf R_CNT_F2

        return

;**********************************
;** Routine: out3_kled           **
;**********************************
; Ausgang 3 blinken K-LED Muster. 
out3_kled
        incf R_CNT_F3,F                ; +1

        movlw T_FLASH_I2               ; duration of a inverted flash #2 
        btfss R_FLAG_F3,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_FLASH_I1              ; duration of a inverted flash #1
        subwf R_CNT_F3,W               ; .. R_CNT_F3 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off
        out_3_off                      ; .. LED off

        movlw T_FLASH3_I               ; Time between 2 Flashes (long time)
        btfss R_FLAG_F3,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK3_I              ; Time between 2 Flashes (short time)
        subwf R_CNT_F3,W               ; if R_CNT_F3 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F3,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        out_3_on                       ; .. LED on
        clrf R_CNT_F3

        return

;**********************************
;** Routine: out4_kled           **
;**********************************
; Ausgang 4 blinken K-LED Muster. 
out4_kled
        incf R_CNT_F4,F                ; +1

        movlw T_FLASH_I2               ; duration of a inverted flash #2 
        btfss R_FLAG_F4,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_FLASH_I1              ; duration of a inverted flash #1
        subwf R_CNT_F4,W               ; .. R_CNT_F4 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off
        out_4_off                      ; .. LED off

        movlw T_FLASH4_I               ; Time between 2 Flashes (long time)
        btfss R_FLAG_F4,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK4_I              ; Time between 2 Flashes (short time)
        subwf R_CNT_F4,W               ; if R_CNT_F4 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F4,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        out_4_on                       ; .. LED on
        clrf R_CNT_F4

        return

;**********************************
;** Routine: out5_kled           **
;**********************************
; Ausgang 5 blinken K-LED Muster. 
out5_kled
        incf R_CNT_F5,F                ; +1

        movlw T_FLASH_I2               ; duration of a flash #2 
        btfss R_FLAG_F5,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_FLASH_I1              ; duration of a flash #1
        subwf R_CNT_F5,W               ; .. R_CNT_F5 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off
        out_5_off                      ; .. FLASH1 off

        movlw T_FLASH5_I               ; Time between 2 Flashes (long time)
        btfss R_FLAG_F5,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK5_I              ; Time between 2 Flashes (short time)
        subwf R_CNT_F5,W               ; if R_CNT_F5 < T_FLASHx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx)
        incf R_FLAG_F5,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> short time)
        out_5_on                       ; .. LED on
        clrf R_CNT_F5

        return

;*********************************
;** Routine: out4_blinker      **
;*********************************
; Ausgang 4 blinken (Blinker) (50%=ON, 50%=OFF)
out4_blinker
        movf R_CNT_F4,W                ; get R_CNT_F4 into W-Register

        decfsz R_CNT_F4,F                ; if T_BILTZ4 is over..
         return                   

        movf GPIO,W
        xorlw b'00000010'              ; toggle OUT4
        movwf GPIO

        movlw T_FLASH4_2               ; .. and
        movwf R_CNT_F4                 ; .. reload coumter

        return                   

;*********************************
;** Routine: out4_Wechselblinken *
;*********************************
; Ausgang 4 + 5 blinken im Wechsel (50%=ON, 50%=OFF)
out4_5_wechsel
        movf R_CNT_F4,W                ; get R_CNT_F4 into W-Register
        decfsz R_CNT_F4,F              ; if T_FLASH8 is over..
         return                   

        incf R_CNT_F4_2,F              ; R_CNT_F4_2 +1
        btfss R_CNT_F4_2,0             ; only every second run ..
         return                   

        movf GPIO,W
        xorlw b'00000010'              ; toggle OUT4
        movwf GPIO

        out_5_off                      ; OUT5 wird..
#ifdef out_inv
        btfsc IO_OUT4                  ; .. invertiert zu
         out_5_on                      ; .. OUT4
#else ; norm
        btfss IO_OUT4                  ; .. invertiert zu
         out_5_on                      ; .. OUT4
#endif

        movlw T_FLASH8                 ; .. and
        movwf R_CNT_F4                 ; .. reload coumter

        return                   

;*******************************
;** Routine: out3_4_frontlight**
;*******************************
; PWM fuer Ausgang 3+4 im Wechsel
out3_4_frontlight
        incfsz R_PWM,F                 ; if R_PWM = 0
         goto out3_4_front_1             ; .. then change R_STATUS
 
        ;***** PWM-Time ON & new Status *****
        ; ***** PWM Wert berechnen *****
        movlw PWM_MIN
         movwf R_PWM

        ; ***** neuen Status *****
        bcf STATUS,C                   ; clear Carry and
        rlf R_STATUS,F                 ; .. rotate left
        btfsc R_STATUS,5               ; if #5
        bsf R_STATUS,3                 ; .. change to #3
        bcf R_STATUS,5                 ; .. and delete #5
out3_4_front_1

        ;***** PWM-Time ON *****
        movlw b'00001000'              ; check status #3
        andwf R_STATUS,W               ; = fade out FRONTLIGHT
        btfsc STATUS,Z
         goto out3_4_001
           out_4_off                   ; FRONTLIGHT1 off
           out_3_on                    ; FRONTLIGHT2 on
out3_4_001

        movlw b'00010000'              ; check status #4
        andwf R_STATUS,W               ; = fade in FRONTLIGHT
        btfsc STATUS,Z
         goto out3_4_002
           out_4_on                    ; FRONTLIGHT1 on
           out_3_off                   ; FRONTLIGHT2 off
out3_4_002

out3_4_front_2
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; return

        clrwdt                         ; clear Watchdog-Timer
        movf R_PWM,W                   ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto out3_4_front_2           ; ... then out3_4_front_2

        ; ***** PWM-Time ready *****
        movlw b'00010000'              ; check status #4
        andwf R_STATUS,W               ; = fade in FRONTLIGHT
        btfsc STATUS,Z
         goto out3_4_003
           out_4_off                   ; FRONTLIGHT1 off
           out_3_on                    ; FRONTLIGHT2 on
out3_4_003
      
        movlw b'00001000'              ; check status #3
        andwf R_STATUS,W               ; = fade out FRONTLIGHT
        btfsc STATUS,Z
         goto out3_4_004
           out_4_on                    ; FRONTLIGHT1 on
           out_3_off                   ; FRONTLIGHT2 off
out3_4_004

        return

;*******************************
;** Routine: out5_frontlight  **
;*******************************
; PWM fuer Ausgang 5
;    *           *           *           *           *           *
;    : /----------\          : /----------\          : /----------\     FRONTLIGHT
;-----/ :        : \----------/ :        : \----------/          : \----
;    :#1   #2    :#3    #4   :#5    #6   :#7    #0   :#1    #2   :#3
;
;Time *: TMR1H <> ZERO and R_COUNT=0
;#1,#5: FRONTLIGHT fade in
;#3,#7: FRONTLIGHT fade out   
;#2,#6: Wartezeit HELL
;#4   : Pausezeit zwischen den LH
;#0   : Wartezeit bei Lichthupe
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms

out5_frontlight
        btfss F_LICHTHUPE
         goto out5_frontlight_fade       ; keine Lichthupe  -> Skip Routine

        ; Lichthupen-Funktion - Spezieller Ablauf fr Mode 32
        movlw b'10101010'                ; check status #7,#5,#3,#1
        andwf R_STATUS_FRONT,W           ; = FADE Status
        btfss STATUS,Z
         goto out5_frontlight_fade       ; .. Jump to Fade Routine

        decfsz R_CNT_LICHTHUPE_1,F     ; Wartezeit auf Lichthupe
         goto out5_front_1             ; ..aber PWM muss gemacht werden
        decfsz R_CNT_LICHTHUPE_2,F     ; Wartezeit auf Lichthupe
         goto out5_front_1             ; ..aber PWM muss gemacht werden

        ; --> aktuell kein FADE Status und Wartezeit abgelaufen -> Werte laden fr nchsten Warte-Status
        btfsc R_STATUS_FRONT,0
         movlw CONST_CNT_LICHTHUPE_HELL   ; Reload Counter
        btfsc R_STATUS_FRONT,2
         movlw CONST_CNT_LICHTHUPE_DUNKEL ; Reload Counter
        btfsc R_STATUS_FRONT,4
         movlw CONST_CNT_LICHTHUPE_HELL2  ; Reload Counter
        btfsc R_STATUS_FRONT,6
         movlw CONST_CNT_LICHTHUPE_WARTE  ; Reload Counter
        movwf R_CNT_LICHTHUPE_2           ; .. laden in Counter 2

        movlw d'255'                   ; R_PWM auf MAX 
        movwf R_PWM                    ; .. sodass nach Wartephase gleich neuer R_STATUS_FRONT berechnet wird

out5_frontlight_fade
        incfsz R_PWM,F                 ; if R_PWM = 0 (Counter fr PWM State)
         goto out5_front_1             ; .. then jump, else change R_STATUS_FRONT
 
        ;***** PWM-Time ON & new Status *****
        ; ***** PWM Wert berechnen *****
        movlw PWM_MIN                  ; R_PWM startet mit dem 
        movwf R_PWM                    ; .. zugelassenen MIN Wert

        ; ***** neuen Status *****
        bcf STATUS,C                   ; clear Carry and
        rlf R_STATUS_FRONT,F           ; .. rotate left

        btfsc STATUS,C                 ; if CARRY
         bsf R_STATUS_FRONT,0          ; .. change to #0

        bcf STATUS,C                   ; clear Carry and
        btfss F_LICHTHUPE              ; wenn nicht F_LICHTHUPE     
         rlf R_STATUS_FRONT,F          ; .. rotate ein 2. mal left
      
out5_front_1
        ; berechnen des PWM-Wertes R_PWMVALUE aus PWM-Counter R_PWM
        movf R_PWM,W                   ; umladen von R_PWM in R_PWMVALUE
        movwf R_PWMVALUE               ; .. (wird verndert)
        movlw d'1'
        btfsc R_STATUS_FRONT,3         ; wenn Ausblenden, dann R_PWMVALUE = 1 - R_PWM
         subwf R_PWMVALUE,F
        btfsc R_STATUS_FRONT,7         ; wenn Ausblenden, dann R_PWMVALUE = 1 - R_PWM
         subwf R_PWMVALUE,F

        ;***** PWM-Time ON *****
        movlw b'11001100'              ; check status #7,#6,#3,#2
        andwf R_STATUS_FRONT,W         ; = fade out/on FRONTLIGHT
        btfss STATUS,Z
         out_5_off                     ; FRONTLIGHT off

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto out5_front_2             ; ... then out5_front_2

        movlw b'00110011'              ; check status #5,#4,#1,#0
        andwf R_STATUS_FRONT,W         ; = fade in/off FRONTLIGHT
        btfss STATUS,Z
         out_5_on                      ; FRONTLIGHT on

out5_front_2  ; ***** wait for PWM-Time ready *****
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; ... return

        clrwdt                         ; clear Watchdog-Timer
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto out5_front_2             ; ... then out5_front_2 (= wait)

        ; ***** PWM-Time ready *****
        movlw b'00110011'              ; check status #5,#4,#1,#0
        andwf R_STATUS_FRONT,W         ; = fade in/off FRONTLIGHT
        btfss STATUS,Z
         out_5_off                     ; FRONTLIGHT off
      
        movlw b'11001100'              ; check status #7,#6,#3,#2
        andwf R_STATUS_FRONT,W         ; = fade out/on FRONTLIGHT
        btfss STATUS,Z
         out_5_on                      ; FRONTLIGHT on

        return


;*********************************
;** Routine: out1_5_aircraft    **
;*********************************
out1_5_aircraft
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH                  ; if 
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
         out_1_off                     ; .. FLASH1 off
         out_2_off                     ; .. FLASH2 off
         out_3_off                     ; .. FLASH3 off
         out_4_off                     ; .. FLASH4 off
         out_5_off                     ; .. FLASH5 off

        movlw T_FLASH6                 ; Time between 2 Flashes (long time)
        btfss R_FLAG_F1,0              ; if short off-Time needed (Strobo double-flash)
         movlw T_BREAK6                ; Time between 2 Flashes (short time)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED on (wenn R_CNT >= T_FLASHx / T_BREAKx)
        incf R_FLAG_F1,F               ; toggle FLAG (#0=1 -> Long time, #1=1 -> R-L)
        movlw b'00000011'
        andwf R_FLAG_F1,F               ; mask unused bits

        movf R_FLAG_F1,F
        btfsc STATUS,Z                 ; if R_FLAG_F1 = ZERO
         out_1_on                      ; .. then TOP LIGHT
        btfsc STATUS,Z                 ; if R_FLAG_F1 = ZERO
         out_2_on                      ; .. then BOT LIGHT
        btfsc R_FLAG_F1,1              ; if R_FLAF_F1 = '...10'
         out_3_on                      ; .. the WING1 Light on
        btfsc R_FLAG_F1,1              ; if R_FLAF_F1 = '...10'
         out_4_on                      ; .. the WING2 Light on

        btfsc R_FLAG_F1,0              ; TAIL has no double Flash
         goto $+3
        btfsc R_FLAG_F1,1              ; if R_FLAF_F1 = '...10'
         out_5_on                      ; .. the TAIL Light on

        clrf R_CNT_F1

        return

;*********************************
;** Routine: out1_2_3_tigis     **
;*********************************
; Ausgang 1/2/3 Tigis / Delphi
out1_2_3_tigis
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH28a               ; if 
        btfsc R_FLAG_F1,3              ; if short Time_B needed 
         movlw T_FLASH28b              ; Time between 2 Flashes (time_b)
        btfsc R_FLAG_F1,4              ; if short Time_C needed 
         movlw T_FLASH28c              ; Time between 2 Flashes (time_c)
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
         out_1_off                     ; .. FLASH1 off
         out_2_off                     ; .. FLASH2 off
         out_3_off                     ; .. FLASH3 off

        movlw T_BREAK28a               ; Time between 2 Flashes (time_a)
        btfsc R_FLAG_F1,2              ; if short Time_C needed 
         movlw T_BREAK28c              ; Time between 2 Flashes (time_c)
        btfsc R_FLAG_F1,3              ; if short Time_C needed 
         movlw T_BREAK28d              ; Time between 2 Flashes (time_d)
        btfsc R_FLAG_F1,4              ; if short Time_C needed 
         movlw T_BREAK28e              ; Time between 2 Flashes (time_d)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; R_CNT >= T_BREAKx
        bcf STATUS,C                   ; Carry lschen fr RLF
        rlf R_FLAG_F1,F                ; shift R_FLAG_F1 
        btfsc R_FLAG_F1,5              ; berlauf nach RLF    
         bsf R_FLAG_F1,0               ; .. abfangen 

        movf R_FLAG_F1,F               ; if R_FLAG_F1 is     
        btfsc STATUS,Z                 ; .. ZERO (no Flag)
         bsf R_FLAG_F1,0               ; .. set flag #0

        ; LED on (wenn R_CNT >= T_BREAKx)
        movlw b'00011001'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_1_on                      ; .. FLASH1 on
  
        movlw b'00011010'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_2_on                      ; .. FLASH1 on

        movlw b'00011100'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_3_on                      ; .. FLASH1 on

        clrf R_CNT_F1

        return

;*********************************
;** Routine: out1_2_3_4-5_tigis **
;*********************************
; Ausgang 1/2/3/4/5 Tigis / Delphi
out1_2_3_4_5_tigis
        incf R_CNT_F1,F                ; +1

        movlw T_FLASH29a               ; if 
        btfsc R_FLAG_F1,5              ; if short Time_B needed 
         movlw T_FLASH29b              ; Time between 2 Flashes (time_b)
        btfsc R_FLAG_F1,6              ; if short Time_C needed 
         movlw T_FLASH29c              ; Time between 2 Flashes (time_c)
        subwf R_CNT_F1,W               ; .. R_CNT_F1 < T_FLASH
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; LED off (wenn R_CNT >= T_FLASH)
         out_1_off                     ; .. FLASH1 off
         out_2_off                     ; .. FLASH2 off
         out_3_off                     ; .. FLASH3 off
         out_4_off                     ; .. FLASH3 off
         out_5_off                     ; .. FLASH3 off

        movlw T_BREAK29a               ; Time between 2 Flashes (time_a)
        btfsc R_FLAG_F1,4              ; if short Time_C needed 
         movlw T_BREAK29c              ; Time between 2 Flashes (time_c)
        btfsc R_FLAG_F1,5              ; if short Time_C needed 
         movlw T_BREAK29d              ; Time between 2 Flashes (time_d)
        btfsc R_FLAG_F1,6              ; if short Time_C needed 
         movlw T_BREAK29e              ; Time between 2 Flashes (time_d)
        subwf R_CNT_F1,W               ; if R_CNT_F1 < T_FLASHx / T_BREAKx
        btfss STATUS,C                 ; .. then 
         return                        ; return

        ; R_CNT >= T_BREAKx
        bcf STATUS,C                   ; Carry lschen fr RLF
        rlf R_FLAG_F1,F                ; shift R_FLAG_F1 
        btfsc R_FLAG_F1,7              ; berlauf nach RLF    
         bsf R_FLAG_F1,0               ; .. abfangen 

        movf R_FLAG_F1,F               ; if R_FLAG_F1 is     
        btfsc STATUS,Z                 ; .. ZERO (no Flag)
         bsf R_FLAG_F1,0               ; .. set flag #0

        ; LED on (wenn R_CNT >= T_BREAKx)
        movlw b'01100001'
        btfsc F_DELFIS                 ; wenn DELFIS, dann kein Doppelplitz
         movlw b'01000001'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_1_on                      ; .. FLASH1 on
  
        movlw b'01100010'
        btfsc F_DELFIS                 ; wenn DELFIS, dann kein Doppelplitz
         movlw b'01000010'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_2_on                      ; .. FLASH1 on

        movlw b'01100100'
        btfsc F_DELFIS                 ; wenn DELFIS, dann kein Doppelplitz
         movlw b'01000100'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_3_on                      ; .. FLASH1 on

        movlw b'01101000'
        btfsc F_DELFIS                 ; wenn DELFIS, dann kein Doppelplitz
         movlw b'01001000'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_4_on                      ; .. FLASH1 on

        movlw b'01110000'
        btfsc F_DELFIS                 ; wenn DELFIS, dann kein Doppelplitz
         movlw b'01010000'
        andwf R_FLAG_F1,W
        btfss STATUS,Z
         out_5_on                      ; .. FLASH1 on

        clrf R_CNT_F1

        return


;*********************************
;** Routine: Wasseranzeige **
;*********************************
; wasseranzeige (klassische Statemachine)
wasseranzeige
        movlw d'1'
        subwf R_CNT_WASSER_1,F
        btfsc STATUS,C
         return

        ; Preload Counter (500 ms)
        movlw d'225'
        movwf R_CNT_WASSER_1

        movlw d'1'
        subwf R_CNT_WASSER_2,F
        btfsc STATUS,C
         return

        ; Preload Counter (1000 ms)
        movlw d'1'
        movwf R_CNT_WASSER_2

        ; State Machine Zeiger INC
        incf R_CNT_F1,F                ; +1

        ; Takt: 1 sekunde

        movlw d'1'                     ; bei "1" bedeutet sofort Start nach Einschalten
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         goto wasseranzeige_all_on     ; RETURN von dort

        ; LED on (wenn R_CNT >= T_BREAKx)
        movlw d'22'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_1_off                     ; Out 1 abschalten
  
        movlw d'24'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_2_off                     ; Out 2 abschalten

        movlw d'26'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_3_off                     ; Out 3 abschalten

        movlw d'28'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_4_off                     ; Out 4 abschalten

        movlw d'30'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         bsf F_WASSER_BLINK            ; Out 0 blinken EIN

        movlw d'90'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         bcf F_WASSER_BLINK            ; Out 0 blinken AUS

        movlw d'91'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_5_on                      ; Out 5 einschalten

        movlw d'95'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_4_on                      ; Out 4 einschalten

        movlw d'100'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_3_on                      ; Out 3 einschalten

        movlw d'105'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_2_on                      ; Out 2 einschalten

        movlw d'110'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         out_1_on                      ; Out 1 einschalten

        movlw d'230'
        subwf R_CNT_F1,W
        btfsc STATUS,Z
         clrf R_CNT_F1                 ; Zyklus neu starten

        btfss F_WASSER_BLINK
         return

wasseranzeige_blinken
        movf GPIO,W
        xorlw b'00000001'             ; GPIO,0 togglen
        movwf GPIO
        return

wasseranzeige_all_on
        out_1_on                      ; Out 1 einschalten
        out_2_on                      ; Out 1 einschalten
        out_3_on                      ; Out 1 einschalten
        out_4_on                      ; Out 1 einschalten
        out_5_on                      ; Out 1 einschalten
        return

;*********************************
;** Routine: out123_ll          **
;*********************************
; Ausgang 1/2/3 Lauflicht (Drehspiegel)
;
;    :#3  #4  #5  #3  #4  #5  #3
;    : /--\        /--\        /-- OUT 1
;-----/    \------/    \------/
;          /--\        /--\
;---------/    \------/    \------ OUT 2
;   --\        /--\        /--\
;      \------/    \------/    \-- OUT 3
;
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_PWM
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_PWM=0
;@2: LED off @ TMR1L > PWM-Value
;@3: LED on, R_PWM increment every 2ms

out123_ll
        movlw d'2'
        btfsc R_MODE,0
         addlw d'2'
        btfsc R_MODE,1
         addlw d'4'
        addwf R_PWM,F
        btfss STATUS,C
         goto out123_ll_1              ; .. then jump (else change R_STATUS)

        ;***** PWM-Time ON & new Status *****
        ; ***** PWM Wert berechnen *****
        movlw PWM_MIN
         movwf R_PWM

        ; ***** neuen Status *****
        bcf STATUS,C                   ; clear Carry and
        rlf R_STATUS,F                 ; .. rotate left
        btfsc R_STATUS,6               ; if #6
         bsf R_STATUS,3                ; .. change to #3
        bcf R_STATUS,6                 ; .. and delete #6

out123_ll_1
        ;***** PWM-Time ON *****
        movlw b'00001000'              ; check status #3
        andwf R_STATUS,W               ; = fade out FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_on_1
           out_1_on                    ; OUT 1 Fade IN
           out_2_off                   ; OUT 2 OFF
           out_3_off                   ; OUT 3 Fade OUT
out123_ll_on_1

        ; TMR1L < PWM-Value
        movlw b'00010000'              ; check status #4
        andwf R_STATUS,W               ; = fade in FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_on_2
           out_1_off                   ; OUT 1 Fade OUT
           out_2_on                    ; OUT 2 Fade IN
           out_3_off                   ; OUT 3 OFF
out123_ll_on_2

        movlw b'00100000'              ; check status #5
        andwf R_STATUS,W               ; = fade in FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_on_3
           out_1_off                   ; OUT 1 OFF
           out_2_off                   ; OUT 2 Fade OUT
           out_3_on                    ; OUT 3 Fade IN
out123_ll_on_3

out123_ll_front_2 ; TMR1L > PWM-Value
        movf TMR1H,F
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; return (warten ist hier nicht mehr ntig)

        clrwdt                         ; clear Watchdog-Timer
        movf R_PWM,W                   ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto out123_ll_front_2        ; ... then out123_ll_front_2

        ; ***** PWM-Time ready *****
        movlw b'00001000'              ; check status #3
        andwf R_STATUS,W               ; = fade in FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_off_1
           out_1_off                   ; OUT 1 Fade IN
           out_2_off                   ; OUT 2 OFF
           out_3_on                    ; OUT 3 Fade OUT
out123_ll_off_1

        movlw b'00010000'              ; check status #4
        andwf R_STATUS,W               ; = fade out FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_off_2
           out_1_on                    ; OUT 1 Fade OUT
           out_2_off                   ; OUT 2 Fade IN
           out_3_off                   ; OUT 3 OFF
out123_ll_off_2

        movlw b'00100000'              ; check status #5
        andwf R_STATUS,W               ; = fade out FRONTLIGHT
        btfsc STATUS,Z
         goto out123_ll_off_3
           out_1_off                   ; OUT 1 OFF
           out_2_on                    ; OUT 2 Fade OUT
           out_3_off                   ; OUT 3 Fade IN
out123_ll_off_3
        return


;*********************************
;** TableRead                   **
;*********************************
 org  h'0300' | Table_Adress_Start    ; Position
; Sprung in die Tabelle. 
; In R_TABLEADRESS_L steht die LOW-Byte Adresse drinn. Es wird nun 
; berechnet, mit welchem ADDWF Wert ans Ziel gesprungen wird.
tableread
        ; PCLATH laden fr Page 3
        movlw h'03'                    ; Page 0x300 
        movwf PCLATH

        ; PCL berechnen
        movlw h'05'                    ; 
        addwf R_TABLEADRESS_L,W        ; Laufzeit fr diese UP ausgleichen 
        movwf PCL                      ; PCL = W = Table_Adress_Start + 0x305

 
;*********************************
;*********************************
;*********************************
;*********************************
;** Effekt Data                 **
;*********************************
;*********************************
;*********************************
;*********************************
        ;*********************************
        ;** Effekt Data / Part 1     *****
        ;*********************************   
;               +----------- #define F_OUT1_2_3_BLINK -> Ausgang 1/2/3 blinken (RTK)
;               |+---------- #define F_OUT1_2_BLINK   -> Ausgang 1 blinken / Zusatz-Optionen: F_OUT12_STROBO_WECHSEL (Wechsel-Strobo mit Out 2) / F_OUT1_STROBO   (Doppelblitz)
;               ||+--------- #define F_OUT3_4_BLINK   -> Ausgang 3 blinken / Zusatz-Optionen: F_OUT34_STROBO_WECHSEL (Wechsel-Strobo mit Out 4) / F_OUT345_STROBO (Doppelblitz)
;               |||+-------- #define F_OUT2_BLINK     -> Ausgang 2 blinken / Zusatz-Optionen: F_OUT2_STROBO   (Doppelblitz)
;               ||||+------- #define F_OUT4_BLINK     -> Ausgang 4 blinken / Zusatz-Optionen: F_OUT345_STROBO (Doppelblitz)
;               |||||+------ #define F_OUT5_BLINK     -> Ausgang 5 blinken / Zusatz-Optionen: F_OUT345_STROBO (Doppelblitz)
;               ||||||+----- #define F_OUT4_5_WECHSEL -> Ausgang 4 + 5 blinken im Wechsel (50%=ON, 50%=OFF)
;               |||||||+---- 
;               ||||||||     
        retlw b'01111100'     ; Effekt Definition #0 Blinkmuster 1
        retlw b'01110100'     ; Effekt Definition #0 Blinkmuster 2
        retlw b'01111000'     ; Effekt Definition #0 Blinkmuster 3
        retlw b'01110000'     ; Effekt Definition #0 Blinkmuster 4
        retlw b'01111100'     ; Effekt Definition #0 Blinkmuster 5
        retlw b'01110100'     ; Effekt Definition #0 Blinkmuster 6
        retlw b'01111000'     ; Effekt Definition #0 Blinkmuster 7
        retlw b'01110000'     ; Effekt Definition #0 Blinkmuster 8
        retlw b'01010100'     ; Effekt Definition #0 Blinkmuster 9
        retlw b'01010000'     ; Effekt Definition #0 Blinkmuster 10
        retlw b'01000100'     ; Effekt Definition #0 Blinkmuster 11
        retlw b'01000000'     ; Effekt Definition #0 Blinkmuster 12
        retlw b'01101100'     ; Effekt Definition #0 Blinkmuster 13
        retlw b'01100100'     ; Effekt Definition #0 Blinkmuster 14
        retlw b'01101000'     ; Effekt Definition #0 Blinkmuster 15
        retlw b'01100000'     ; Effekt Definition #0 Blinkmuster 16
        retlw b'01100100'     ; Effekt Definition #0 Blinkmuster 17
        retlw b'01100000'     ; Effekt Definition #0 Blinkmuster 18
        retlw b'01100000'     ; Effekt Definition #0 Blinkmuster 19
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 20
        retlw b'10001100'     ; Effekt Definition #0 Blinkmuster 21
        retlw b'10000010'     ; Effekt Definition #0 Blinkmuster 22
        retlw b'01111100'     ; Effekt Definition #0 Blinkmuster 23
        retlw b'01110100'     ; Effekt Definition #0 Blinkmuster 24
        retlw b'01110000'     ; Effekt Definition #0 Blinkmuster 25
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 26
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 27
        retlw b'00001100'     ; Effekt Definition #0 Blinkmuster 28
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 29
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 30
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 31
        retlw b'01110000'     ; Effekt Definition #0 Blinkmuster 32
        retlw b'00000100'     ; Effekt Definition #0 Blinkmuster 33
        retlw b'00000100'     ; Effekt Definition #0 Blinkmuster 34
        retlw b'00000100'     ; Effekt Definition #0 Blinkmuster 35
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 36
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 37
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 38
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 39
        retlw b'00000000'     ; Effekt Definition #0 Blinkmuster 40
                 
        ;*********************************
        ;** Effekt Data / Part 2     *****
        ;*********************************
;               +----------- #define F_OUT4_BLINKER      -> unabhngiger Effekt fr Out 4, F_OUT_BLINK darf nicht gesetzt sein
;               |+---------- #define F_OUT5_FRONTLIGHT   -> Lichthupe Out 5, Zusatz-Option: F_LICHTHUPE (einmalige Lichthupe)
;               ||+--------- #define F_OUT3_4_FRONTLIGHT -> unabhngiger Effekt fr Out 3/4 (PWM fuer Ausgang 3+4 im Wechsel)
;               |||+-------- #define F_OUT1_KLED         -> unabhngiger Effekt fr Out 1
;               ||||+------- #define F_OUT2_KLED         -> unabhngiger Effekt fr Out 2
;               |||||+------ #define F_OUT3_KLED         -> unabhngiger Effekt fr Out 3
;               ||||||+----- #define F_OUT4_KLED         -> unabhngiger Effekt fr Out 4
;               |||||||+---- #define F_OUT5_KLED         -> unabhngiger Effekt fr Out 5
;               ||||||||     
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 1
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 2
        retlw b'01000000'     ; Effekt Definition #1 Blinkmuster 3
        retlw b'11000000'     ; Effekt Definition #1 Blinkmuster 4
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 5
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 6
        retlw b'01000000'     ; Effekt Definition #1 Blinkmuster 7
        retlw b'11000000'     ; Effekt Definition #1 Blinkmuster 8
        retlw b'00100000'     ; Effekt Definition #1 Blinkmuster 9
        retlw b'00100001'     ; Effekt Definition #1 Blinkmuster 10
        retlw b'00100000'     ; Effekt Definition #1 Blinkmuster 11
        retlw b'00100001'     ; Effekt Definition #1 Blinkmuster 12
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 13
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 14
        retlw b'01000000'     ; Effekt Definition #1 Blinkmuster 15
        retlw b'11000000'     ; Effekt Definition #1 Blinkmuster 16
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 17
        retlw b'00000001'     ; Effekt Definition #1 Blinkmuster 18
        retlw b'01000000'     ; Effekt Definition #1 Blinkmuster 19
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 20
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 21
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 22
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 23
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 24
        retlw b'00000011'     ; Effekt Definition #1 Blinkmuster 25
        retlw b'00011111'     ; Effekt Definition #1 Blinkmuster 26
        retlw b'10011101'     ; Effekt Definition #1 Blinkmuster 27
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 28
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 29
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 30
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 31
        retlw b'11000000'     ; Effekt Definition #1 Blinkmuster 32
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 33
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 34
        retlw b'10000000'     ; Effekt Definition #1 Blinkmuster 35
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 36
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 37
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 38
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 39
        retlw b'00000000'     ; Effekt Definition #1 Blinkmuster 40

        ;*********************************
        ;** Effekt Data / Part 3     *****
        ;*********************************
;               +----------- #define F_OUT1_5_AIRCRAFT        -> unabhngiger Effekt fr Out 1/2/3/4/5
;               |+---------- #define F_OUT1_2_3_TIGIS         -> unabhngiger Effekt fr Out 1/2/3
;               ||+--------- #define F_OUT1_2_3_4_5_TIGIS     -> unabhngiger Effekt fr Out 1/2/3/4
;               |||+-------- #define F_WASSERANZEIGE          -> unabhngiger Effekt fr Out 1/2/3/4/5
;               ||||+------- #define F_DELFIS                 -> Zusatz-Option zu F_OUT1_2_3_4_5_TIGIS
;               |||||+------ #define F_OUT12_STROBO_WECHSEL   -> Zusatz-Option zu F_OUT1_2_BLINK (Wechsel-Strobo mit Out 2)
;               ||||||+----- #define F_OUT34_STROBO_WECHSEL   -> Zusatz-Option zu F_OUT3_4_BLINK (Wechsel-Strobo mit Out 4)
;               |||||||+---- #define F_LICHTHUPE              -> Zusatz-Option zu F_OUT5_FRONTLIGHT (Lichthupe)
;               ||||||||     
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 1
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 2
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 3
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 4
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 5
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 6
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 7
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 8
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 9
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 10
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 11
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 12
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 13
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 14
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 15
        retlw b'00000100'     ; Effekt Definition #2 Blinkmuster 16
        retlw b'00000110'     ; Effekt Definition #2 Blinkmuster 17
        retlw b'00000110'     ; Effekt Definition #2 Blinkmuster 18
        retlw b'00000110'     ; Effekt Definition #2 Blinkmuster 19
        retlw b'10000000'     ; Effekt Definition #2 Blinkmuster 20
        retlw b'00000110'     ; Effekt Definition #2 Blinkmuster 21
        retlw b'00000110'     ; Effekt Definition #2 Blinkmuster 22
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 23
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 24
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 25
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 26
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 27
        retlw b'01000010'     ; Effekt Definition #2 Blinkmuster 28
        retlw b'00100000'     ; Effekt Definition #2 Blinkmuster 29
        retlw b'00101000'     ; Effekt Definition #2 Blinkmuster 30
        retlw b'00010000'     ; Effekt Definition #2 Blinkmuster 31
        retlw b'00000001'     ; Effekt Definition #2 Blinkmuster 32
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 33
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 34
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 35
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 36
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 37
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 38
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 39
        retlw b'00000000'     ; Effekt Definition #2 Blinkmuster 40

        ;*********************************
        ;** Effekt Data / Part 4     *****
        ;*********************************
;               +----------- #define F_OUT1_STROBO   (Zusatz-Option zu F_OUT1_BLINK)
;               |+---------- #define F_OUT2_STROBO   (Zusatz-Option zu F_OUT2_BLINK)
;               ||+--------- #define F_OUT345_STROBO (Zusatz-Option zu F_OUT3_4_BLINK / F_OUT4_BLINK / F_OUT5_BLINK)
;               |||+-------- #define F_OUT123_LL     (Drehspiegel - unabhngiger Effekt fr Out 1/2/3)
;               ||||+------- #define F_                   
;               |||||+------ #define F_                   
;               ||||||+----- #define F_                   
;               |||||||+---- #define F_                   
;               ||||||||     
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 1
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 2
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 3
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 4
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 5
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 6
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 7
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 8
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 9
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 10
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 11
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 12
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 13
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 14
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 15
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 16
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 17
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 18
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 19
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 20
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 21
        retlw b'11100000'     ; Effekt Definition #3 Blinkmuster 22
        retlw b'10000000'     ; Effekt Definition #3 Blinkmuster 23
        retlw b'10000000'     ; Effekt Definition #3 Blinkmuster 24
        retlw b'10000000'     ; Effekt Definition #3 Blinkmuster 25
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 26
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 27
        retlw b'00100000'     ; Effekt Definition #3 Blinkmuster 28
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 29
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 30
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 31
        retlw b'00100000'     ; Effekt Definition #3 Blinkmuster 32
        retlw b'00110000'     ; Effekt Definition #3 Blinkmuster 33
        retlw b'00110000'     ; Effekt Definition #3 Blinkmuster 34
        retlw b'00110000'     ; Effekt Definition #3 Blinkmuster 35
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 36
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 37
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 38
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 39
        retlw b'00000000'     ; Effekt Definition #3 Blinkmuster 40

         END       
                   

